// Copyright Sebastian Wiesner <sebastian@swsnr.de>
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.
//
// Alternatively, the contents of this file may be used under the terms
// of the GNU General Public License Version 2 or later, as described below:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
import GLib from "gi://GLib";
import Gio from "gi://Gio";

/**
 * Whether `reason` is a GLib error denoting a cancelled IO operation.
 */
export const isGioErrorCancelled = (reason) => reason instanceof GLib.Error &&
    reason.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.CANCELLED);

/**
 * Run an IO operation as cancellable promise.
 *
 * @param run Create a cancellable promise from the given cancellable.
 * @returns The promise and a handle to cancel it
 */
export const runCancellable = (run) => {
    const cancellable = Gio.Cancellable.new();

    const promise = run(cancellable)
        .then((value) => cancellable.is_cancelled()
        ? { result: "cancelled" }
        : { result: "completed", value })
        .catch((error) => {
        if (isGioErrorCancelled(error) || cancellable.is_cancelled()) {
            // If the error represents a cancelled operation, or if we hit another
            // error while the operation was already cancelled, just swallow the
            // error and return a cancelled result, under the assumption, that the
            // caller is never interested in errors of cancelled operations.
            return { result: "cancelled" };
        }
        else {
            throw error;
        }
    });

    return [promise, cancellable];
};

/**
 * Denotes an IO error.
 *
 * Use this error to wrap GIO errors which do not have a more specific context.
 */
export class IOError extends Error {
}
