// Copyright Sebastian Wiesner <sebastian@swsnr.de>
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.
//
// Alternatively, the contents of this file may be used under the terms
// of the GNU General Public License Version 2 or later, as described below:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
import Gio from "gi://Gio";

const isStoredMetadata = (value) => {
    return (value !== null &&
        typeof value === "object" &&
        Object.hasOwn(value, "uri") &&
        typeof value["uri"] === "string" &&
        Object.hasOwn(value, "metadata") &&
        Object.hasOwn(value["metadata"], "title"));
};

const parseStoredMetadata = (value) => {
    try {
        const parsed = JSON.parse(value);
        if (isStoredMetadata(parsed)) {
            return parsed;
        }
        else {
            console.warn("Stored metadata not valid", parsed);

            return null;
        }
    }
    catch (error) {
        console.warn("Failed to parse stored metadata", value, error);

        return null;
    }
};

/**
 * Store the metadata for the current image.
 */
export class ImageMetadataStore {
    settings;

    /**
     * Create a new store.
     *
     * @param settings The settings to store the image metadata in
     */
    constructor(settings) {
        this.settings = settings;
    }

    /**
     * Store metadata for the given image.
     */
    storedMetadataForImage(image) {
        const stored = {
            metadata: image.metadata,
            uri: image.file.get_uri(),
        };
        this.settings.set_string("current-metadata", JSON.stringify(stored));
    }

    /**
     * Load the current image from the metadata store.
     *
     * @returns The stored image or null if no image was stored or the store was invalid
     */
    loadFromMetadata() {
        const stored = parseStoredMetadata(this.settings.get_string("current-metadata"));
        if (stored !== null) {
            return {
                metadata: stored.metadata,
                file: Gio.File.new_for_uri(stored.uri),
            };
        }
        else {
            return null;
        }
    }
}
