// Copyright Sebastian Wiesner <sebastian@swsnr.de>
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.
//
// Alternatively, the contents of this file may be used under the terms
// of the GNU General Public License Version 2 or later, as described below:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
/**
 * Whether `n` is an element.
 */
export const isElement = (n) => n !== null && typeof n !== "undefined" && typeof n !== "string";

/**
 * An error which occurred while processing this XML.
 */
export class XmlError extends Error {
}

/**
 * Run a parser against an input and return the result.
 *
 * @param parser The parser to run
 * @param s The input string
 * @param options saxes options
 * @returns A list of all fragment nodes parsed out of `s`.
 */
const runParser = (parser, s, options) => {
    const toplevels = [];
    const elementStack = [];
    parser.on("error", (error) => {
        throw new XmlError(error.message, { cause: error });
    });
    parser.on("opentag", (node) => {
        const element = {
            name: node.name,
            attributes: node.attributes,
            children: [],
        };
        if (elementStack.length === 0) {
            toplevels.push(element);
        }
        else {
            elementStack[elementStack.length - 1]?.children.push(element);
        }
        elementStack.push(element);
    });
    parser.on("text", (text) => {
        if (!(options?.removeWhitespaceTextNodes && text.trim().length === 0)) {
            elementStack[elementStack.length - 1]?.children.push(text);
        }
    });
    parser.on("cdata", (data) => {
        if (!(options?.removeWhitespaceTextNodes && data.trim().length === 0)) {
            elementStack[elementStack.length - 1]?.children.push(data);
        }
    });
    parser.on("closetag", () => {
        elementStack.pop();
    });
    parser.write(s).close();

    return toplevels;
};

/**
 * Parse an XML document in `s`.
 *
 * @returns The single root element
 */
export const parse = (Parser) => (s, options) => {
    const roots = runParser(new Parser(), s, options);
    if (roots.length === 1 && isElement(roots[0])) {
        return roots[0];
    }
    else {
        throw new XmlError("Expected exactly one root element");
    }
};

/**
 * Parse XML fragments out of `s`.
 *
 * @returns An array with all fragments.
 */
export const parseFragments = (Parser) => (s, options) => runParser(new Parser({ fragment: true }), s, options);

/** Get immediate child elements with the given name. */
export const childrenByName = (e, name) => e.children.filter((n) => isElement(n) && n.name === name);

/** Find the first immediate child with the given name. */
export const childByName = (e, name) => e.children.find((n) => isElement(n) && n.name === name);

/** Get the recursive inner text of the given element. */
export const innerText = (e) => e.children
    .map((n) => {
    if (isElement(n)) {
        return innerText(n);
    }
    else {
        return n;
    }
})
    .join("");
