// Copyright Sebastian Wiesner <sebastian@swsnr.de>
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.
//
// Alternatively, the contents of this file may be used under the terms
// of the GNU General Public License Version 2 or later, as described below:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
import GLib from "gi://GLib";
import { gettext as _ } from "resource:///org/gnome/shell/extensions/extension.js";
import { getJSON } from "../network/http.js";
import metadata from "./metadata/wikimedia.js";
import { NoPictureTodayError } from "../source/errors.js";

/**
 * Assemble a copyright description from the image metadata.
 *
 * @param image The featured imaged
 */
const getCopyrightText = (image) => {
    const { artist, license, credit } = image;
    if (artist?.text && license?.type && credit?.text) {
        return `${artist.text} (${credit.text}, ${license.type})`;
    }
    else if (artist?.text && license?.type) {
        return `${artist.text} (${license.type})`;
    }
    else if (artist?.text) {
        return artist.text;
    }
    else if (license?.type) {
        return license.type;
    }
    else {
        // We do not know anything about the copyright here, so let's default to
        // some "all rights reserved" blurp.
        return _("Unknown, all rights reserved");
    }
};

const getFeaturedContent = async (session, date, cancellable) => {
    // Extract the language code from the locale, and fall back to English.
    const locales = GLib.get_language_names_with_category("LC_MESSAGES");
    const languageCode = locales[0]?.split("_")[0] ?? "en";
    const urlDate = date.format("%Y/%m/%d");
    if (urlDate === null) {
        throw new Error("Failed to format timestamp");
    }
    const url = `https://api.wikimedia.org/feed/v1/wikipedia/${languageCode}/featured/${urlDate}`;
    console.log(`Fetching featured content from ${url}`);
    const response = await getJSON(session, url, cancellable);

    return response;
};

const getImages = async (session, cancellable) => {
    const date = GLib.DateTime.new_now_local();
    const featuredContent = await getFeaturedContent(session, date, cancellable);
    if (!featuredContent.image) {
        throw new NoPictureTodayError(metadata);
    }
    const { description, file_page, image, title } = featuredContent.image;
    const pubdate = date.format("%Y-%m-%d");
    if (pubdate === null) {
        throw new Error(`Formatting GLib.DateTime returned null?`);
    }
    const copyright = getCopyrightText(featuredContent.image);

    const imageToDownload = {
        imageUrl: image.source,
        pubdate,
        metadata: {
            // Drop File: prefix and file extension from image title.
            title: title.replaceAll(/(^File:|.[^.]+$)/g, ""),
            description: description?.text ?? null,
            copyright,
            url: file_page,
        },
    };

    return [imageToDownload];
};

/**
 * A source for images from Wikimedia.
 */
export const source = {
    metadata,
    getImages: {
        type: "simple",
        getImages,
    },
};

export default source;
